/*
COPYRIGHT (C) 2008 EPISERVER AB

THIS FILE IS PART OF WIKIX.
WIKIX IS FREE SOFTWARE: YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU LESSER GENERAL PUBLIC LICENSE VERSION v2.1 AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION.

WIKIX IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU LESSER GENERAL PUBLIC LICENSE FOR MORE DETAILS.
YOU SHOULD HAVE RECEIVED A COPY OF THE GNU LESSER GENERAL PUBLIC LICENSE ALONG WITH WIKIX. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
*/

using System;
using System.Web;
using System.Web.UI;
using EPiServer.Core;
using EPiServer.Web.WebControls;
using EPiServer.Filters;
using EPiServer.Research.Wiki.Core;

namespace EPiServer.Research.Wiki.Pages
{
    public partial class Search : WikiPageBase
    {
        private string _query;
       
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            
          
                _query = ((string)HttpContext.Current.Items["Query"]) ?? Request.QueryString["quicksearchquery"];
                if (!String.IsNullOrEmpty(_query))
                {
                    _query = _query.Replace('+', ' ');
                    SearchDataSource.SearchQuery = _query;
                    
                    SearchDataSource.Filter += new FilterEventHandler(this.Filter);
                    SearchRes.Visible = true;
                    PageDataCollection pdc = (PageDataCollection) SearchDataSource.Select(DataSourceSelectArguments.Empty);

                    if (cbAdvancedSearch.Checked)
                    {
                        pdc = RemoveOnPageName(pdc);
                    }

                    SearchRes.DataSource = pdc;

                    if (pdc.Count == 0)
                    {
                        NoSearchResult.Visible = true;
                        hlNewArticle.Text = "Create New Article " + _query + " ?";
                        hlNewArticle.NavigateUrl = WikiManager.Instance.GetWikiRootFriendlyUrl(CurrentPageLink) + _query;
                    }
                    else
                    {
                        NoSearchResult.Visible = false;
                        SearchDataSource.DataBind();
                        SearchRes.DataBind();
                    }

                }
                
                caption.DataBind();
                cbAdvancedSearch.DataBind();
                
        }

        protected string GetTitle(object o)
        {
            PageData pd = (o as PageData);

            if (pd == null)
            {
                return String.Empty;
            }
            if (pd.PageTypeName == WikiManager.WIKIARTICLENAME)
            {
                return pd.PageName;
            }
            return WikiManager.Instance.GetWikiArticle(pd).PageName + " > " + pd.PageName;
        }

        public void Filter(object sender, FilterEventArgs e)
        {
            PageDataCollection pages = e.Pages;
            PageData pageData;
            for (int i = 0; i < pages.Count; i++)
            {
                pageData = pages[i];
                if (pageData.PageTypeName == WikiManager.WIKIDISCUSSIONNAME)
                {
                    pages.RemoveAt(i);
                    i--;
                }
            }
        }
        private PageDataCollection RemoveOnPageName(PageDataCollection pagedatacollection)
        {
            PageDataCollection pages = pagedatacollection;
            PageData pageData;
            for (int i = 0; i < pages.Count; i++)
            {
                pageData = pages[i];
                if (!String.Equals(pageData.PageName, _query, StringComparison.OrdinalIgnoreCase))
                {
                    pages.RemoveAt(i);
                    i--;
                }
            }

            return pages;
        }
 
    }
}
