/*
COPYRIGHT (C) 2008 EPISERVER AB

THIS FILE IS PART OF WIKIX.
WIKIX IS FREE SOFTWARE: YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU LESSER GENERAL PUBLIC LICENSE VERSION v2.1 AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION.

WIKIX IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU LESSER GENERAL PUBLIC LICENSE FOR MORE DETAILS.
YOU SHOULD HAVE RECEIVED A COPY OF THE GNU LESSER GENERAL PUBLIC LICENSE ALONG WITH WIKIX. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
*/

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using EPiServer;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.Web.WebControls;
using EPiServer.Research.Wiki.Core;

namespace EPiServer.Research.Wiki.Units
{
    public partial class SectionMenu : EPiServer.UserControlBase
    {
        public string CssClass { get; set; }

        public PageReference ActivePage { get; set; }

        public bool IsArticle { get; set; }

        public bool CanMoveUp()
        {
            return GetPage(ActivePage ?? CurrentPage.PageLink).QueryDistinctAccess(EPiServer.Security.AccessLevel.Edit)
                &&
                WikiManager.Instance.CanMoveUpSection(ActivePage);
        }

        public bool CanMoveDown()
        {
            return GetPage(ActivePage ?? CurrentPage.PageLink).QueryDistinctAccess(EPiServer.Security.AccessLevel.Edit)
                &&
                WikiManager.Instance.CanMoveDownSection(ActivePage);
        }

        public bool CanDelete()
        {
            return GetPage(ActivePage ?? CurrentPage.PageLink).QueryDistinctAccess(EPiServer.Security.AccessLevel.Delete)
                && (!EditManager.Instance.IsLocked(ActivePage ?? CurrentPage.PageLink));
        }

        public bool IsBeingEdited()
        {
            return EditManager.Instance.IsLocked(ActivePage ?? CurrentPage.PageLink);
        }

        public bool CanEdit()
        {
            return GetPage(ActivePage ?? CurrentPage.PageLink).QueryDistinctAccess(EPiServer.Security.AccessLevel.Edit)
                && (!EditManager.Instance.IsLocked(ActivePage ?? CurrentPage.PageLink));
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(CssClass)) CssClass = "ArticleIconMenu";
        }

        protected string EditLink()
        {
            return CurrentPage.LinkURL + "&edit=" + (ActivePage ?? CurrentPage.PageLink).ToString();
        }

        protected string AddLink()
        {
            return CurrentPage.LinkURL + "&add=" + (ActivePage ?? CurrentPage.PageLink).ToString();
        }

        /// <summary>
        /// Handles the event of a page move
        /// </summary>
        /// <param name="Sender"></param>
        /// <param name="e"></param>
        public void HandleMove(object Sender, CommandEventArgs e)
        {
            if (e.CommandName == "MoveUp")
            {
                WikiManager.Instance.MoveUpSection(PageReference.Parse((string)e.CommandArgument));
                Response.Redirect(CurrentPage.LinkURL);
            }
            else if (e.CommandName == "MoveDown")
            {
                WikiManager.Instance.MoveDownSection(PageReference.Parse((string)e.CommandArgument));
                Response.Redirect(CurrentPage.LinkURL);
            }
        }

        public void HandleDelete(object Sender, CommandEventArgs e)
        {
            if (e.CommandName == "Delete")
            {
                WikiManager.Instance.DeleteSection(PageReference.Parse((string)e.CommandArgument));
                Response.Redirect(CurrentPage.LinkURL);
            }
        }
    }
}